/**
 *
 */
package jp.agentec.sinaburocast.action.admin.support;

import java.io.InputStream;
import java.net.URLEncoder;
import java.util.List;

import javax.annotation.Resource;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.csv.OrganizationCsv;
import jp.agentec.sinaburocast.entity.UploadError;
import jp.agentec.sinaburocast.form.admin.support.OrganizationShelfForm;
import jp.agentec.sinaburocast.service.OrganizationService;
import jp.agentec.sinaburocast.service.UploadErrorService;
import jp.agentec.sinaburocast.service.csv.CsvService;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.apache.struts.util.TokenProcessor;
import org.seasar.framework.util.FileUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;
import org.seasar.struts.util.ResponseUtil;

/**
 * 団体一括登録
 * @author choi-c
 *
 */
public class OrganizationShelfRegistAction extends AbstractAction {
	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	@Resource
	public OrganizationShelfForm organizationShelfForm;

	/**
	 * ジェネリクスを変えることでCSV関連Entity対応出来る。
	 */
	public CsvService<OrganizationCsv> csvService;

	public OrganizationService organizationService;

	public UploadErrorService uploadErrorService;

	//ファイルタイプID
	private final String type="3";
	/**
	 * 初期表示
	 */
	@Execute(validator = false)
	public String index() {
		//エラーファイルダウンロード
		organizationShelfForm.uploadError = uploadErrorService.findByType(type);
		//団体一括登録
		return "/admin/support/organizationShelfRegist.html";
	}

	/**
	 * 市民活動団体一括登録
	 * @throws Exception
	 */
	@Execute(validator = true,input="/admin/support/organizationShelfRegist/")
	public String organizationUpload()throws Exception {


		String csvData = null;
		InputStream in = null;

		if(!organizationShelfForm.file.getFileName().toUpperCase().endsWith(".TSV")){
			addError("errors.E040");
			return "/admin/support/organizationShelfRegist/";
		}
		//チェック
		try {
			in = organizationShelfForm.file.getInputStream();
			//csvData = IOUtils.toString(in, "UTF-8");
			//バイナリ形式チェック
            byte[] b = new byte[1];
            while (in.read(b, 0, 1) > 0) {
                if (b[0] == 0) {
        			addError("errors.E042");
        			return "/admin/support/organizationShelfRegist/";
                }
            }
//			csvData = IOUtils.toString(in, "Shift_JIS");

		}catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			IOUtils.closeQuietly(in);
		}
		//チェック後値設定
		try {
			in = organizationShelfForm.file.getInputStream();
			//20170508 SJIS文字化け対応
			csvData = IOUtils.toString(in, "Windows-31J");
			//csvData = IOUtils.toString(in, "Shift_JIS");
		}catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			IOUtils.closeQuietly(in);

		}

		StringBuffer isOk   = new StringBuffer("OK");

		List<OrganizationCsv> organizationCsvList=null;
		try {
			organizationCsvList = csvService.parseAll(csvData, OrganizationCsv.class,isOk);
		} catch (org.seasar.s2csv.csv.exception.CSVFormatException e) {
			addError("errors.E039");
			return "/admin/support/organizationShelfRegist/";
		}catch(NumberFormatException e){
			addError("errors.E039");
			return "/admin/support/organizationShelfRegist/";
		}




		if(CollectionUtils.isEmpty(organizationCsvList) ){
			addError("errors.E037");
			return "/admin/support/organizationShelfRegist/";
		}


		UploadError uploadError = new UploadError();
		uploadError.filePath = PropertyUtil.getProperty(SinaburoConstant.filePath.UPLOAD_ERROR_FILE_PATH);
		uploadError.filePath+="organization_upload_error.tsv";
		uploadError.type=type;//市民団体一括アップロード
		uploadError.status =  StringUtil.equals(isOk.toString(), "NG") ? "NG" : "OK";

		uploadErrorService.deleteInsertByType(uploadError, getAdmin());
		organizationShelfForm.uploadError = uploadErrorService.findByType(type);

		/**結果ファイル作成*/
		//20170508 SJIS文字化け対応
		String outStr = SinaburoUtil.addDoubleStr(csvService.getContents(organizationCsvList, OrganizationCsv.class).replace("\"", ""));
		outStr = SinaburoUtil.toSJIS(outStr);
		FileUtil.write(uploadError.filePath, outStr.getBytes("Windows-31J"));
		//FileUtil.write(uploadError.filePath,
		//	SinaburoUtil.addDoubleStr(csvService.getContents(organizationCsvList, OrganizationCsv.class).replace("\"", ""))
		//	.getBytes("Shift_JIS"));

		/**データチェックエラー(1件でも)時には「市民活動団体一括登録」処理を行わない*/
		if(StringUtil.equals(isOk.toString(), "NG")){
	    	// トークン発行
			TokenProcessor.getInstance().saveToken(request);
			addError(SinaburoViewUtil.getMessage("errors.E036"));
			return "/admin/support/organizationShelfRegist/";
		}

		/**一括登録*/
		organizationService.organizationShelfRegist(organizationCsvList, getAdmin());

		//団体一括登録
		return "/admin/support/organizationShelfRegistEnd.html";
	}

	/**
	 * 最後に登録したファイルの結果ファイルをダウンロード
	 * @throws Exception
	 */
	@Execute(validator = false)
	public String resultFileDownLoad()throws Exception {
		 UploadError uploadError = uploadErrorService.findByType(type);
		 jp.agentec.sinaburocast.common.io.FileUtil.fileDownload(uploadError.filePath, "Shift_JIS");

		 organizationShelfForm.uploadError = uploadErrorService.findByType(type);

		//団体一括登録
		return null;
	}

	/**
	 * フォーマットファイルダウンロード
	 * @throws Exception
	 */
	@Execute(validator = false)
	public String formatFileDownLoad()throws Exception {
		//String header="\"団体番号\"\t\"団体名\"\t\"結果\"\t\"理由\"\r\n";
		String header="\"団体番号\"\t\"団体名\"\r\n";
		
		//20170508 SJIS文字化け対応
		header = SinaburoUtil.toSJIS(header);
		ResponseUtil.download(URLEncoder.encode("organization_upload.tsv", "UTF-8"), header.getBytes("Windows-31J"));
		//ResponseUtil.download(URLEncoder.encode("organization_upload.tsv", "UTF-8"), header.getBytes("Shift_JIS"));

		//団体一括登録
		return null;
	}



}
